clear all;

% synthetic data
Nobs=10000;
Ntypes=2;
dobs = zeros(Nobs,Ntypes);
d1mean=5;
sigmad1=0.5;
dobs(:,1) = random( 'Normal', d1mean, sigmad1, Nobs, 1 );
d2mean=6;
sigmad2=1;
dobs(:,2) = random( 'Normal', d2mean, sigmad2, Nobs, 1 );

% scatter plot of data
figure(1);
clf;
set(gca,'LineWidth',2);
hold on;
axis( [0, 10, 0, 10] );
plot( dobs(:,1), dobs(:,2), 'k.', 'LineWidth', 2 );
xlabel('d1');
ylabel('d2');


% estimate of mean, variance assuming Normality
d1meanest = mean(dobs(:,1));
d2meanest = mean(dobs(:,2));
plot( d1meanest, d2meanest, 'ro', 'LineWidth', 2 );
sigmad1est = std(dobs(:,1));
sigmad2est = std(dobs(:,2));
plot( [d1meanest-sigmad1est, d1meanest+sigmad1est]', [d2meanest, d2meanest]', 'r-', 'LineWidth', 2 );
plot( [d1meanest, d1meanest]', [d2meanest-sigmad2est, d2meanest+sigmad2est]', 'r-', 'LineWidth', 2 );
Cd = cov(dobs(:,1:2));
cov12 = Cd(1,2);
fprintf('covariance %f\n', cov12 );
[R, P, Rlow, Rhigh] = corrcoef(dobs(:,1:2));
r12 = R(1,2);
r12low = Rlow(1,2);
r12high = Rhigh(1,2);
fprintf('corrcoeff %f < %f < %f (95%%)\n', r12low, r12, r12high );


% MATLAB documnetation says:
% The confidence bounds are based on an asymptotic normal
%     distribution of 0.5*log((1+R)/(1-R)), with an approximate variance equal
%     to 1/(N-3).  These bounds are accurate for large samples when X has a
%     multivariate normal distribution.

% histogram bins
Nbins = 101;
dbinmin = 0;
dbinmax = 10;
Dd = (dbinmax-dbinmin)/(Nbins-1);
dbins = dbinmin + Dd*[0:Nbins-1]';

% histogram
d1hist = hist( dobs(:,1), dbins )';
d2hist = hist( dobs(:,2), dbins )';

% plot histogram of data
figure(2);
clf;
subplot(2,1,1);
set(gca,'LineWidth',2);
hold on;
d1histmax = max(d1hist);
axis( [dbinmin, dbinmax, 0, d1histmax] );
plot( dbins, d1hist, 'k-', 'LineWidth', 2 );
plot( [d1meanest-sigmad1est, d1meanest-sigmad1est]', [0, 0.05*d1histmax]', 'r-', 'LineWidth', 4 );
plot( [d1meanest+sigmad1est, d1meanest+sigmad1est]', [0, 0.05*d1histmax]', 'r-', 'LineWidth', 4 );
plot( [d1meanest, d1meanest]', [0, 0.05*d1histmax]', 'r-', 'LineWidth', 4 );
xlabel('d');
ylabel('counts');
subplot(2,1,2);
set(gca,'LineWidth',2);
hold on;
d2histmax=max(d2hist);
axis( [dbinmin, dbinmax, 0, d2histmax] );
plot( dbins, d2hist, 'k-', 'LineWidth', 2 );
plot( [d2meanest-sigmad2est, d2meanest-sigmad2est]', [0, 0.05*d2histmax]', 'r-', 'LineWidth', 4 );
plot( [d2meanest+sigmad2est, d2meanest+sigmad2est]', [0, 0.05*d2histmax]', 'r-', 'LineWidth', 4 );
plot( [d2meanest, d2meanest]', [0, 0.05*d2histmax]', 'r-', 'LineWidth', 4 );
xlabel('d');
ylabel('counts');

% normalize to probability density function
p1 = d1hist / ( Dd * sum(d1hist) );
p2 = d2hist / ( Dd * sum(d2hist) );

% plot pdf
figure(3);
clf;
subplot(2,1,1);
set(gca,'LineWidth',2);
hold on;
p1max = max(p1);
axis( [dbinmin, dbinmax, 0, p1max] );
plot( dbins, p1, 'k-', 'LineWidth', 2 );
plot( [d1meanest-sigmad1est, d1meanest-sigmad1est]', [0, 0.05*p1max]', 'r-', 'LineWidth', 4 );
plot( [d1meanest+sigmad1est, d1meanest+sigmad1est]', [0, 0.05*p1max]', 'r-', 'LineWidth', 4 );
plot( [d1meanest, d1meanest]', [0, 0.05*p1max]', 'r-', 'LineWidth', 4 );
xlabel('d');
ylabel('p(d)');
subplot(2,1,2);
set(gca,'LineWidth',2);
hold on;
p2max=max(p2);
axis( [dbinmin, dbinmax, 0, p2max] );
plot( dbins, p2, 'k-', 'LineWidth', 2 );
plot( [d2meanest-sigmad2est, d2meanest-sigmad2est]', [0, 0.05*p2max]', 'r-', 'LineWidth', 4 );
plot( [d2meanest+sigmad2est, d2meanest+sigmad2est]', [0, 0.05*p2max]', 'r-', 'LineWidth', 4 );
plot( [d2meanest, d2meanest]', [0, 0.05*p2max]', 'r-', 'LineWidth', 4 );
xlabel('d');
ylabel('p(d)');

% estimate mean, std from pdf
d1meanestPDF = Dd * sum( dbins .* p1 );
sigmad1estPDF = sqrt( Dd * sum( (dbins-d1meanestPDF).^2 .* p1 ) );
subplot(2,1,1);
plot( [d1meanestPDF-sigmad1estPDF, d1meanestPDF-sigmad1estPDF]', [0, 0.05*p1max]', 'g-', 'LineWidth', 2 );
plot( [d1meanestPDF+sigmad1estPDF, d1meanestPDF+sigmad1estPDF]', [0, 0.05*p1max]', 'g-', 'LineWidth', 2 );
plot( [d1meanestPDF, d1meanestPDF]', [0, 0.05*p1max]', 'g-', 'LineWidth', 2 );
d2meanestPDF = Dd * sum( dbins .* p2 );
sigmad2estPDF = sqrt( Dd * sum( (dbins-d2meanestPDF).^2 .* p2 ) );
subplot(2,1,2);
plot( [d2meanestPDF-sigmad2estPDF, d2meanestPDF-sigmad2estPDF]', [0, 0.05*p2max]', 'g-', 'LineWidth', 2 );
plot( [d2meanestPDF+sigmad2estPDF, d2meanestPDF+sigmad2estPDF]', [0, 0.05*p2max]', 'g-', 'LineWidth', 2 );
plot( [d2meanestPDF, d2meanestPDF]', [0, 0.05*p2max]', 'g-', 'LineWidth', 2 );

% cumulative distribution
P1 = Dd * cumsum(p1);
P2 = Dd * cumsum(p2);

% plot cumulative distribution
figure(4);
clf;
subplot(2,1,1);
set(gca,'LineWidth',2);
hold on;
axis( [dbinmin, dbinmax, 0, 1.05] );
plot( dbins, P1, 'k-', 'LineWidth', 2 );
xlabel('d');
ylabel('P(d)');
subplot(2,1,2);
set(gca,'LineWidth',2);
hold on;
axis( [dbinmin, dbinmax, 0, 1.05] );
plot( dbins, P2, 'k-', 'LineWidth', 2 );
xlabel('d');
ylabel('P(d)');

% 95 percent confidence
d1low = dbins( find( P1>0.025, 1 ) );
d1high = dbins( find( P1>0.975, 1 ) );
subplot(2,1,1);
plot( [d1low, d1low]', [0, 0.05]', 'r-', 'LineWidth', 4 );
plot( [d1high, d1high]', [0, 0.05]', 'r-', 'LineWidth', 4 );
plot( [d1meanestPDF-2*sigmad1estPDF, d1meanestPDF-2*sigmad1estPDF]', [0, 0.05]', 'g-', 'LineWidth', 2 );
plot( [d1meanestPDF+2*sigmad1estPDF, d1meanestPDF+2*sigmad1estPDF]', [0, 0.05]', 'g-', 'LineWidth', 2 );
d2low = dbins( find( P2>0.025, 1 ) );
d2high = dbins( find( P2>0.975, 1 ) );
subplot(2,1,2);
plot( [d2low, d2low]', [0, 0.05]', 'r-', 'LineWidth', 4 );
plot( [d2high, d2high]', [0, 0.05]', 'r-', 'LineWidth', 4 );
plot( [d2meanestPDF-2*sigmad2estPDF, d2meanestPDF-2*sigmad2estPDF]', [0, 0.05]', 'g-', 'LineWidth', 2 );
plot( [d2meanestPDF+2*sigmad2estPDF, d2meanestPDF+2*sigmad2estPDF]', [0, 0.05]', 'g-', 'LineWidth', 2 );
