clear all;

% model: d = m1 exp( m2 * x );
M=2;
mtrue = [1, -0.2]';

% auxillary variable x
N=101;
xmin = 0;
xmax = 10;
Dx = (xmax-xmin)/(N-1);
x = xmin + Dx*[0:N-1]';

% data
dtrue = mtrue(1) * exp( mtrue(2) * x );
sigmad = 0.3;
dobs = dtrue + random('Normal', 0, sigmad, N, 1 );

% plot of true and observed data
figure(1);
clf;
set(gca,'LineWidth',2);
hold on;
axis( [xmin, xmax, -1, 2] );
plot( x, zeros(N,1), 'k-', 'LineWidth', 2 );
plot( x, dtrue, 'r-', 'LineWidth', 2 );
plot( x, dobs, 'ko', 'LineWidth', 2 );
xlabel('x');
ylabel('d');

% log transformation
% d = m1 * exp( m2 * x )
% log(d) = log(m1) + m2 * x
% d'     = m1'     + m2' * x

% least squares fit
k=find( dobs > 0 ); % throw out negative data
Nk = length(k);
G = [ ones(Nk,1), x(k) ];
dp = log(dobs(k));
mp = (G'*G)\(G'*dp);
dppre = G*mp;
dest = exp(dppre);
plot( x(k), dest, 'g-', 'LineWidth', 2 );
mest(1) = exp(mp(1));
mest(2) = mp(2);

% non-linear least squares
% d = m1 * exp( m2 * x )
% dd/dm1 = exp( m2 * x );
% dd/dm2 = m1 * x * exp( m2 * x )
% G = [ dd/dm1, dd/dm2]

mg = [ 1.5, -0.3 ]';
Lmg = sqrt(mg'*mg);
Niter = 10;
for iter=[1:Niter]
    e = exp( mg(2) * x );
    G = [e, mg(1)*x.*e];
    dg = mg(1) * exp( mg(2) * x );
    dm = (G'*G)\(G'*(dobs-dg));
    test = 1e-6;
    if( sqrt(dm'*dm) < test )
        break
    end
    mg = mg + dm;
end

dg = mg(1) * exp( mg(2) * x );
plot( x, dg, 'b-', 'LineWidth', 2 );

fprintf('m\n');
fprintf('true:            %f   %f\n', mtrue(1), mtrue(2) );
fprintf('log transformed: %f   %f\n', mest(1), mest(2) );
fprintf('linearized:      %f   %f\n', mg(1), mg(2) );




